__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from marketplacecli.utils import org_utils
from ussclicore.utils import printer
from ussclicore.utils import generics_utils
from marketplacecli.utils import marketplace_utils
from marketplace.objects.marketplace import *


class VendorCmds(Cmd, CoreGlobal):
    """Manage vendors : list vendors and more"""

    cmd_name = "vendor"

    def __init__(self):
        super(VendorCmds, self).__init__()

    def arg_list(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                   description="List the vendors registered to the marketplace")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_list(self, args):
        try:
            # add arguments
            do_parser = self.arg_list()
            do_args = do_parser.parse_args(args.split())

            # call UForge API
            printer.out("Getting vendors ...")
            org = org_utils.org_get(self.api, do_args.org)
            vendors = self.api.Orgs(org.dbId).Vendors().Getall()

            table = Texttable(200)
            table.set_cols_align(["c", "c", "c"])
            table.header(["Active", "Name", "Email"])
            sorted_vendor_list = generics_utils.order_list_object_by(vendors.vendors.vendor, "name")
            for vendor in sorted_vendor_list:
                table.add_row(["" if vendor.inactive else "X", vendor.name, vendor.email])
            print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_list(self):
        do_parser = self.arg_info()
        do_parser.print_help()



__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from marketplace.objects.marketplace import *
from marketplacecli.utils import org_utils
from ussclicore.utils import printer
from role_entitlements_cmds import RoleEntitlementCmds
from marketplacecli.utils import marketplace_utils
import pyxb


class RoleCmds(Cmd, CoreGlobal):
    """Manage platform roles"""

    cmd_name = "role"

    def __init__(self):
        self.subCmds = {}
        self.generate_sub_commands()
        super(RoleCmds, self).__init__()

    def generate_sub_commands(self):
        entitlements_cmds = RoleEntitlementCmds()
        self.subCmds[entitlements_cmds.cmd_name] = entitlements_cmds

    def arg_list(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                   description="List all the roles for a given organization.  If not organization is provided the default organization is used.")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_list(self, args):
        try:
            org_name = None
            if args:
                do_parser = self.arg_list()
                try:
                    do_args = do_parser.parse_args(args.split())
                except SystemExit as e:
                    return
                org_name = do_args.org

            # call UForge API
            printer.out("Getting all the roles for the organization...")
            org = org_utils.org_get(self.api, org_name)
            all_roles = self.api.Orgs(org.dbId).Roles().Getall(None)

            table = Texttable(200)
            table.set_cols_align(["c", "c"])
            table.header(["Name", "Description"])
            for role in all_roles.roles.role:
                table.add_row([role.name, role.description])
            print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_list(self):
        do_parser = self.arg_list()
        do_parser.print_help()

    def arg_info(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " info", add_help=True,
                                   description="Prints out all the details of a specified role within an organization")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="name of the role")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_info(self, args):
        try:
            # add arguments
            do_parser = self.arg_info()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            # call UForge API
            printer.out("Getting role [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            all_roles = self.api.Orgs(org.dbId).Roles().Getall(None)

            selected_role = None
            for role in all_roles.roles.role:
                if role.name == do_args.name:
                    selected_role = role
                    break

            if selected_role is None:
                printer.out("No role [" + do_args.name + "]...")
                return 1

            printer.out("Name: " + selected_role.name)
            printer.out("Description: " + selected_role.description)
            table = Texttable(200)
            table.set_cols_align(["l", "l"])
            table.header(["Entitlement", "Description"])
            for entitlement in selected_role.entitlements.entitlement:
                table.add_row([entitlement.name, entitlement.description])
            print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_info(self):
        do_parser = self.arg_info()
        do_parser.print_help()

    def arg_create(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " create", add_help=True,
                                   description="Create a new empty role in the specified organization")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True,
                               help="the name of the subscription profile to create")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--description', dest='description', required=False,
                              help="a role description for which the current command should be executed")
        optional.add_argument('--entitlements', dest='entitlements', required=False,
                              help="a list of entitlements to be added to this role during creation (example: --entitlements ent1 ent2 ent3). For a list of available entitlements, run the command: uforge entitlement list --org <org name>")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_create(self, args):
        try:
            # add arguments
            do_parser = self.arg_create()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Creating role [" + do_args.name + "] ...")
            org = org_utils.org_get(self.api, do_args.org)

            new_role = role()
            new_role.name = do_args.name
            if do_args.description:
                new_role.description = do_args.description

            self.api.Orgs(org.dbId).Roles().Create(new_role)
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_create()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_create(self):
        do_parser = self.arg_create()
        do_parser.print_help()

    def arg_delete(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " delete", add_help=True,
                                   description="Delete a role from the specified organization")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="name of the role")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_delete(self, args):
        try:
            # add arguments
            do_parser = self.arg_delete()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Deleting role [" + do_args.name + "] ...")
            org = org_utils.org_get(self.api, do_args.org)
            self.api.Orgs(org.dbId).Roles(do_args.name).Delete()
            printer.out("Role [" + do_args.name + "] deleted")
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_delete(self):
        do_parser = self.arg_delete()
        do_parser.print_help()



__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from marketplacecli.utils import org_utils
from ussclicore.utils import printer
from ussclicore.utils import generics_utils
from marketplace.objects.marketplace import *
from subscriptionprofile_admins import SubscriptionProfileAdmins
from subscriptionprofile_roles import SubscriptionProfileRoles
from marketplacecli.utils import marketplace_utils
import pyxb


class SubscriptionProfileCmds(Cmd, CoreGlobal):
    """Manage subscription profiles : list profile, create profiles, update profiles"""

    cmd_name = "subscription"

    def __init__(self):
        self.subCmds = {}
        self.generate_sub_commands()
        super(SubscriptionProfileCmds, self).__init__()

    def generate_sub_commands(self):
        subscriptionprofile_roles_cmds = SubscriptionProfileRoles()
        self.subCmds[subscriptionprofile_roles_cmds.cmd_name] = subscriptionprofile_roles_cmds
        subscriptionprofile_admin_cmds = SubscriptionProfileAdmins()
        self.subCmds[subscriptionprofile_admin_cmds.cmd_name] = subscriptionprofile_admin_cmds

    def arg_list(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                   description="List all the subscription profiles for a given organization.  If no organization is provided the default organization is used.")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_list(self, args):
        try:
            org_name = None
            if args:
                do_parser = self.arg_list()
                try:
                    do_args = do_parser.parse_args(args.split())
                except SystemExit as e:
                    return
                org_name = do_args.org

            # call UForge API
            printer.out("Getting all the subscription profiles for organization ...")
            org = org_utils.org_get(self.api, org_name)
            print org
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(None)

            table = Texttable(200)
            table.set_cols_align(["c", "c", "c", "c"])
            table.header(["Name", "Code", "Active", "description"])
            for subscription in subscriptions.subscriptionProfiles.subscriptionProfile:
                table.add_row([subscription.name, subscription.code, "X" if subscription.active else "",
                               subscription.description])
            print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_list(self):
        do_parser = self.arg_list()
        do_parser.print_help()

    def arg_info(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " info", add_help=True,
                                   description="Get detailed information on a subscription profile within an organization.")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of the subscription profile")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_info(self, args):
        try:
            # add arguments
            do_parser = self.arg_info()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            # call UForge API
            printer.out("Getting subscription profile with name [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            table = Texttable(200)
            table.set_cols_align(["l", "l"])
            table.header(["Info", "Value"])
            subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]
            table.add_row(["Name", subscription.name])
            table.add_row(["Code", subscription.code])
            table.add_row(["Active", "Yes" if subscription.active else "No"])
            if subscription.roles.role:
                table.add_row(["Roles", ""])
                for role in subscription.roles.role:
                    table.add_row(["", role.name])
            else:
                table.add_row(["Roles", "None"])
            if subscription.admins.admin:
                table.add_row(["Administrators", ""])
                for admin in subscription.admins.admin:
                    table.add_row(["", admin.loginName])
            else:
                table.add_row(["Administrators", "None"])
            print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_info(self):
        do_parser = self.arg_info()
        do_parser.print_help()

    def arg_create(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " create", add_help=True,
                                   description="Create a new subscription profile within an organization.Create a new subscription profile within an organization.")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True,
                               help="the name of the subscription profile to create")
        mandatory.add_argument('--code', dest='code', required=True,
                               help="the code of the subscription profile to create")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--description', dest='description', required=False,
                              help="the description of the subscription profile to create")
        optional.add_argument('--active', dest='active', required=False,
                              help="flag to make the subscription profile active.")
        optional.add_argument('--admins', dest='admins', nargs='+', required=False,
                              help="admin users to be added to the subscription profile that can use the subscription profile to create a user (users separated by spaces)")
        optional.add_argument('--roles', dest='roles', nargs='+', required=False,
                              help="roles to be added to the subscription profile")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_create(self, args):
        try:
            # add arguments
            do_parser = self.arg_create()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            # call UForge API
            printer.out("Creating subscription profile [" + do_args.name + "] ...")
            org = org_utils.org_get(self.api, do_args.org)

            # create a user manually
            new_subscription_profile = subscriptionProfile()
            new_subscription_profile.name = do_args.name
            new_subscription_profile.code = do_args.code
            if do_args.description:
                new_subscription_profile.description = do_args.description
            if do_args.active:
                new_subscription_profile.active = do_args.active

            new_subscription_profile.admins = pyxb.BIND()
            if do_args.admins:
                for a in do_args.admins:
                    new_admin = user()
                    new_admin.loginName = a
                    new_subscription_profile.admins.append(new_admin)

            new_subscription_profile.roles = pyxb.BIND()
            if do_args.roles:
                for a in do_args.roles:
                    new_role = role()
                    new_role.name = a
                    new_subscription_profile.roles.append(new_role)

            # Send the create user request to the server
            new_subscription_profile = self.api.Orgs(org.dbId).Subscriptions().Add(new_subscription_profile)

            if new_subscription_profile is None:
                printer.out("No information about the new subscription profile available", printer.ERROR)
            else:
                table = Texttable(200)
                table.set_cols_align(["c", "c", "c"])
                table.header(
                    ["Name", "Code", "Active"])
                table.add_row([new_subscription_profile.name, new_subscription_profile.code,
                               "X" if new_subscription_profile.active else ""])
                print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_create()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_create(self):
        do_parser = self.arg_create()
        do_parser.print_help()

    def arg_delete(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " delete", add_help=True,
                                   description="Delete a subscription profile from an organization.")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True,
                               help="the name of the subscription profile to delete")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_delete(self, args):
        try:
            # add arguments
            do_parser = self.arg_delete()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Deleting subscription profile [" + do_args.name + "] ...")
            org = org_utils.org_get(self.api, do_args.org)

            # call UForge API
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            if len(subscriptions.subscriptionProfiles.subscriptionProfile) == 1:
                subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]
                self.api.Orgs(org.dbId).Subscriptions(subscription.dbId).Remove(None)
                printer.out("Subscription profile [" + do_args.name + "] deleted")
            else:
                printer.out("Unable to delete subscription profile [" + do_args.name + "]")
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_delete(self):
        do_parser = self.arg_delete()
        do_parser.print_help()

    def arg_update(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " create", add_help=True,
                                   description="Creates a new subscription profile")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True,
                               help="the name of the subscription profile to update")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--description', dest='description', required=False,
                              help="the description of the subscription profile to update")
        optional.add_argument('--active', dest='active', required=False,
                              help="flag to make the subscription profile active.")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_update(self, args):
        try:
            # add arguments
            do_parser = self.arg_update()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting subscription profile with name [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            # call UForge API
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            if subscriptions.total == 1:
                subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]
                updated_subscription = subscriptionProfile()
                updated_subscription.name = subscription.name
                updated_subscription.code = subscription.code
                if do_args.description:
                    updated_subscription.description = do_args.description
                if do_args.active:
                    updated_subscription.active = True
                else:
                    updated_subscription.active = False
                printer.out("Updating subscription profile with name [" + do_args.name + "] ...")
                # call UForge API
                self.api.Orgs(org.dbId).Subscriptions(subscription.dbId).Update(updated_subscription)
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_update()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_update(self):
        do_parser = self.arg_update()
        do_parser.print_help()



__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import printer
from marketplace.objects.marketplace import user
from marketplace.objects.marketplace import users
from marketplacecli.utils import org_utils
from marketplacecli.utils import marketplace_utils
import pyxb


class SubscriptionProfileAdmins(Cmd, CoreGlobal):
    """Manage subscription profile admins"""

    cmd_name = "admin"

    def __init__(self):
        super(SubscriptionProfileAdmins, self).__init__()

    def arg_add(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " add", add_help=True,
                                   description="Add a user as a subscription profile administrator.")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of the subscription profile")
        mandatory.add_argument('--admins', dest='admins', nargs='+', required=True,
                               help="the login names to add as new subscription profile administrators.")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_add(self, args):
        try:
            # add arguments
            do_parser = self.arg_add()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting subscription profile with name [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            if subscriptions is None:
                printer.out("No subscription with name [" + do_args.name + "]")
            else:
                subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]

                # Create the list of administrators
                admins = users()
                admins.users = pyxb.BIND()

                # Copy the list of current administrators
                for admin in subscription.admins.admin:
                    already_admin = user()
                    already_admin.loginName = admin.loginName
                    admins.users.append(already_admin)

                for e in do_args.admins:
                    new_admin = user()
                    new_admin.loginName = e
                    admins.users.append(new_admin)

                # call UForge API
                self.api.Orgs(org.dbId).Subscriptions(subscription.dbId).Admins.Update(admins)
                printer.out("Some users added as administrators of subscription profile [" + do_args.name + "]...")
                return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_add()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_add(self):
        do_parser = self.arg_add()
        do_parser.print_help()

    def arg_remove(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " remove", add_help=True,
                                   description="Remove one or more entitlements to a role within the specified organization")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of the subscription profile")
        mandatory.add_argument('--admins', dest='admins', nargs='+', required=True,
                               help="the login names of the users to be removed from the the subscription profile administrators")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_remove(self, args):
        try:
            # add arguments
            do_parser = self.arg_remove()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting subscription profile with name [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            if subscriptions is None:
                printer.out("No subscription with name [" + do_args.name + "]")
            else:
                subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]

                # Create the list of administrators
                admins = users()
                admins.users = pyxb.BIND()

                # Copy the list of current administrators - Remove the user selected in the input
                for admin in subscription.admins.admin:
                    if admin.loginName not in do_args.admins:
                        already_admin = user()
                        already_admin.loginName = admin.loginName
                        admins.users.append(already_admin)

                # call UForge API
                self.api.Orgs(org.dbId).Subscriptions(subscription.dbId).Admins.Update(admins)
                printer.out("Some users removed as administrators of subscription profile [" + do_args.name + "]...")
                return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_remove()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_remove(self):
        do_parser = self.arg_remove()
        do_parser.print_help()



__author__ = "UShareSoft"

from marketplacecli.exceptions.UForgeException import UForgeException
from marketplacecli.utils import marketplace_utils


def org_get(api, name, on_error_raise=True):
    try:
        org = None
        if name is None:
            org = api.Orgs("default").Get()
            return org
        else:
            orgs = api.Orgs().Getall(None)
            for o in orgs.orgs.org:
                if o.name == name:
                    org = o
    except Exception as e:
        marketplace_utils.print_uforge_exception(e)

    if org is None and on_error_raise:
        raise UForgeException("Unable to find organization")
    return org



__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from marketplace.objects.marketplace import *
from marketplacecli.utils import org_utils
from ussclicore.utils import printer
from ussclicore.utils import generics_utils
from marketplacecli.utils import marketplace_utils
import pyxb


class RoleEntitlementCmds(Cmd, CoreGlobal):
    """Manage roles entitlements"""

    cmd_name = "entitlement"

    def __init__(self):
        super(RoleEntitlementCmds, self).__init__()

    def arg_add(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " add", add_help=True,
                                   description="Add one or more entitlements to a role within the specified organization")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of role")
        mandatory.add_argument('--entitlements', dest='entitlements', nargs='+', required=True,
                               help="the login name to add as a new subscription profile administrator.")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_add(self, args):
        try:
            # add arguments
            do_parser = self.arg_add()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting role [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            all_roles = self.api.Orgs(org.dbId).Roles().Getall(None)

            old_role = None
            for r in all_roles.roles.role:
                if r.name == do_args.name:
                    old_role = r
                    break

            if old_role is None:
                printer.out("No role [" + do_args.name + "]...")
                return 1

            new_role = role()
            new_role.name = old_role.name
            new_role.description = old_role.description
            new_role.entitlements = pyxb.BIND()

            for r in old_role.entitlements.entitlement:
                already_entitlement = entitlement()
                already_entitlement.name = r.name
                new_role.entitlements.append(already_entitlement)

            for e in do_args.entitlements:
                new_entitlement = entitlement()
                new_entitlement.name = e
                new_role.entitlements.append(new_entitlement)

            self.api.Orgs(org.dbId).Roles().Update(new_role)
            printer.out("Role [" + do_args.name + "] updated with new entitlements.")
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_add()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_add(self):
        do_parser = self.arg_add()
        do_parser.print_help()

    def arg_remove(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " remove", add_help=True,
                                   description="Remove one or more entitlements to a role within the specified organization")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of role")
        mandatory.add_argument('--entitlements', dest='entitlements', nargs='+', required=True,
                               help="the login name to remove as a new subscription profile administrator.")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_remove(self, args):
        try:
            # add arguments
            do_parser = self.arg_remove()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting role [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            all_roles = self.api.Orgs(org.dbId).Roles().Getall(None)

            old_role = None
            for r in all_roles.roles.role:
                if r.name == do_args.name:
                    old_role = r
                    break

            if old_role is None:
                printer.out("No role [" + do_args.name + "]...")
                return 1

            new_role = role()
            new_role.name = old_role.name
            new_role.description = old_role.description
            new_role.entitlements = pyxb.BIND()

            for r in old_role.entitlements.entitlement:
                if r.name not in do_args.entitlements:
                    already_entitlement = entitlement()
                    already_entitlement.name = r.name
                    new_role.entitlements.append(already_entitlement)

            self.api.Orgs(org.dbId).Roles().Update(new_role)
            printer.out("Role [" + do_args.name + "] updated with new entitlements.")
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_remove()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_remove(self):
        do_parser = self.arg_remove()
        do_parser.print_help()



__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import printer
from marketplacecli.utils import marketplace_utils


class EntitlementCmds(Cmd, CoreGlobal):
    """Manage entitlements: list them all"""

    cmd_name = "entitlement"

    def __init__(self):
        super(EntitlementCmds, self).__init__()

    def arg_list(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                   description="List all the entitlements in UForge.")
        return do_parser

    def do_list(self, args):
        try:
            # call UForge API
            printer.out("Getting all the entitlements for the platform ...")
            all_entitlements = self.api.Entitlements.Getall(None)

            table = Texttable(200)
            table.set_cols_align(["l", "l"])
            table.header(["Name", "Description"])
            for entitlement in all_entitlements.entitlements.entitlement:
                table.add_row([entitlement.name, entitlement.description])
            print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_list(self):
        do_parser = self.arg_list()
        do_parser.print_help()



__author__ = "UShareSoft"

import traceback

from marketplacecli.exceptions.UForgeException import UForgeException
from marketplace.objects.marketplace import UForgeError

from ussclicore.utils import printer


def is_uforge_exception(e):
    if len(e.args) >= 1 and type(e.args[0]) is UForgeError:
        return True


def get_uforge_exception(e):
    if len(e.args) >= 1 and type(e.args[0]) is UForgeError:
        return "UForge Error '" + str(e.args[0].statusCode) + "' with method: " + e.args[0].requestMethod + " " + \
               e.args[0].requestUri + "\n" + "Message:\n\t" + e.args[0].localizedErrorMsg.message


def print_uforge_exception(e):
    if type(e) is UForgeException:
        printer.out(e.value, printer.ERROR)
    elif len(e.args) >= 1 and type(e.args[0]) is UForgeError:
        printer.out(get_uforge_exception(e), printer.ERROR)
    else:
        traceback.print_exc()


def handle_uforge_exception(e):
    print_uforge_exception(e)
    return 1


def handle_bad_parameters(cmd, e):
    printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
